/***************************************************************************
                     playerwidget.cpp  -  description
                        -------------------
begin                : Mit Nov 20 2002
copyright            : (C) 2002 by Mark Kretschmann
email                :
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "amarokbutton.h"
#include "browserwin.h"
#include "effectwidget.h"
#include "playerapp.h"
#include "playerwidget.h"
#include "playlistwidget.h"
#include "visdistortwidget.h"
#include "viswidget.h"

#include "debugareas.h"

#include <qbitmap.h>
#include <qclipboard.h>
#include <qevent.h>
#include <qfont.h>
#include <qframe.h>
#include <qimage.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <qpainter.h>
#include <qpalette.h>
#include <qpixmap.h>
#include <qpoint.h>
#include <qpopupmenu.h>
#include <qpushbutton.h>
#include <qrect.h>
#include <qslider.h>
#include <qstring.h>
#include <qtoolbutton.h>
#include <qtooltip.h>
#include <qwidget.h>

#include <kaction.h>
#include <kdebug.h>
#include <kglobalaccel.h>
#include <khelpmenu.h>
#include <kkeydialog.h>
#include <klistview.h>
#include <klocale.h>
#include <kpopupmenu.h>
#include <kstandarddirs.h>
#include <ksystemtray.h>
#include <kmessagebox.h>

#include <dcopclient.h>

#define VIS_COUNT 2

// CLASS AmarokSlider ------------------------------------------------------------

AmarokSlider::AmarokSlider( QWidget *parent ) : QSlider( parent )
{}


AmarokSlider::~AmarokSlider()
{}


void AmarokSlider::mousePressEvent( QMouseEvent *e )
{
    float newVal;

    if ( orientation() == QSlider::Horizontal )
        newVal = static_cast<float>( e->x() ) / static_cast<float>( width() ) * maxValue();
    else
        newVal = static_cast<float>( e->y() ) / static_cast<float>( height() ) * maxValue();

    int intVal = static_cast<int>( newVal );

    if ( ( intVal < value() - 10 ) || ( intVal > value() + 10 ) )
    {
        pApp->m_bSliderIsPressed = true;
        setValue( intVal );
        emit sliderReleased();
    }

    QSlider::mousePressEvent( e );
}


// CLASS AmarokSystray ------------------------------------------------------------

// AmarokSystray
// FIXME Move implementation to separate sourcefile
AmarokSystray::AmarokSystray( PlayerWidget *child, KActionCollection *ac ) : KSystemTray( child )
{
    setPixmap( kapp->miniIcon() );

    // berkus: Since it doesn't come to you well, i'll explain it here:
    // We put playlist actions last because: 1) you don't want to accidentally
    // switch amaroK off by pushing rmb on tray icon and then suddenly lmb on the
    // bottom item. 2) if you do like in case 1) the most frequent operation is to
    // change to next track, so it must be at bottom. [usability]
    contextMenu() ->clear();
    contextMenu() ->insertTitle( kapp->miniIcon(), kapp->caption() );

    ac->action( "options_configure" )->plug( contextMenu() );
    contextMenu() ->insertItem( i18n( "&Help" ), ( new KHelpMenu( this, KGlobal::instance() ->aboutData() ) ) ->menu() );
    ac->action( "file_quit" )->plug( contextMenu() );

    contextMenu() ->insertSeparator();

    contextMenu() ->insertItem( QIconSet( locate( "data", "amarok/images/hi16-action-noatunback.png" ) ), i18n( "[&Z] Prev" ), kapp, SLOT( slotPrev() ) );
    contextMenu() ->insertItem( QIconSet( locate( "data", "amarok/images/hi16-action-noatunplay.png" ) ), i18n( "[&X] Play" ), kapp, SLOT( slotPlay() ) );
    contextMenu() ->insertItem( QIconSet( locate( "data", "amarok/images/hi16-action-noatunpause.png" ) ), i18n( "[&C] Pause" ), kapp, SLOT( slotPause() ) );
    contextMenu() ->insertItem( QIconSet( locate( "data", "amarok/images/hi16-action-noatunstop.png" ) ), i18n( "[&V] Stop" ), kapp, SLOT( slotStop() ) );
    contextMenu() ->insertItem( QIconSet( locate( "data", "amarok/images/hi16-action-noatunforward.png" ) ), i18n( "[&B] Next" ), kapp, SLOT( slotNext() ) );

    // don't love them just yet
    setAcceptDrops( false );
}


void AmarokSystray::wheelEvent( QWheelEvent *e )
{
    if ( e->orientation() == Horizontal )
        return ;

    switch ( e->state() )
    {
        case ShiftButton:
            static_cast<PlayerApp *>( kapp ) ->m_pPlayerWidget->wheelEvent( e );
            break;
        default:
            if ( e->delta() > 0 )
                static_cast<PlayerApp *>( kapp ) ->slotNext();
            else
                static_cast<PlayerApp *>( kapp ) ->slotPrev();
            break;
    }

    e->accept();
}



void AmarokSystray::mousePressEvent( QMouseEvent *e )
{
  if( e->button() == MidButton )
    static_cast<PlayerApp *>(kapp)->slotPause();
  else
    KSystemTray::mousePressEvent( e );
}


// CLASS PlayerWidget ------------------------------------------------------------

PlayerWidget::PlayerWidget( QWidget *parent, const char *name )
        : QWidget( parent, name )
        , DCOPObject( "player" )
{
    setName( "PlayerWidget " );
    setCaption( "amaroK" );
    setFixedSize( 310, 155 );
    setPaletteForegroundColor( pApp->m_fgColor );
    m_pPopupMenu = NULL;
    m_pPlayObjConfigWidget = NULL;
    m_nowPlaying = "";

    m_pActionCollection = new KActionCollection( this );
    KStdAction::aboutApp( pApp, SLOT( slotShowAbout() ), m_pActionCollection );
    KStdAction::helpContents( pApp, SLOT( slotShowHelp() ), m_pActionCollection );
    KStdAction::tipOfDay( pApp, SLOT( slotShowTip() ), m_pActionCollection );
    KStdAction::keyBindings( this, SLOT( slotConfigShortcuts() ), m_pActionCollection );
    KStdAction::keyBindings( this, SLOT( slotConfigGlobalShortcuts() ), m_pActionCollection,
                "options_configure_global_keybinding" )->setText( i18n( "Configure Global Shortcuts" ) );
    KStdAction::preferences( pApp, SLOT( slotShowOptions() ), m_pActionCollection );
    KStdAction::quit( pApp, SLOT( quit() ), m_pActionCollection );
    KStdAction::copy( this, SLOT( slotConfigGlobalShortcuts() ), m_pActionCollection,
                "copy_clipboard" )->setText( i18n( "Copy Current Title to Clipboard" ) );
    
//     new KAction( "Copy Current Title to Clipboard", CTRL + Key_C,
//                  this, SLOT( slotCopyClipboard() ), m_pActionCollection, "copy_clipboard" );

    m_oldBgPixmap.resize( size() );

    if ( paletteBackgroundPixmap() )
        m_oldBgPixmap = *paletteBackgroundPixmap();
    else
        m_oldBgPixmap.fill( pApp->m_bgColor );

    setPaletteBackgroundPixmap( QPixmap( locate( "data", "amarok/images/amaroKonlyHG_w320.jpg" ) ) );

    m_pFrame = new QFrame( this );
    m_pFrameButtons = new QFrame( this );
    m_pFrameButtons->setPaletteBackgroundPixmap( m_oldBgPixmap );

    m_pSlider = new AmarokSlider( this );
    m_pSlider->setOrientation( QSlider::Horizontal );
    m_pSlider->setPageStep( 1 );
    m_pSlider->setValue( 0 );
    m_pSlider->setTracking( true );
    m_pSlider->setFocusPolicy( QWidget::NoFocus );

    m_pSliderVol = new AmarokSlider( this );
    m_pSliderVol->setOrientation( QSlider::Vertical );
    m_pSliderVol->setPageStep( 1 );
    m_pSliderVol->setValue( 0 );
    m_pSliderVol->setTracking( true );
    m_pSliderVol->setFocusPolicy( QWidget::NoFocus );

    QString pathStr( locate( "data", "amarok/images/hi16-action-noatunback.png" ) );

    if ( pathStr == QString::null )
        QMessageBox::warning( this, i18n( "amaroK Error" ),
                              i18n( "Error: Could not find icons. Did you forget make install?" ),
                              QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton );

    m_pButtonPrev = new QPushButton( m_pFrameButtons );
    m_pButtonPrev->setFocusPolicy( QWidget::NoFocus );
    m_pButtonPrev->setPixmap( locate( "data", "amarok/images/hi16-action-noatunback.png" ) );
    m_pButtonPrev->setFlat( true );

    m_pButtonPlay = new QPushButton( m_pFrameButtons );
    m_pButtonPlay->setFocusPolicy( QWidget::NoFocus );
    m_pButtonPlay->setPixmap( locate( "data", "amarok/images/hi16-action-noatunplay.png" ) );
    m_pButtonPlay->setToggleButton( true );
    m_pButtonPlay->setFlat( true );

    m_pButtonPause = new QPushButton( m_pFrameButtons );
    m_pButtonPause->setFocusPolicy( QWidget::NoFocus );
    m_pButtonPause->setPixmap( locate( "data", "amarok/images/hi16-action-noatunpause.png" ) );
    m_pButtonPause->setFlat( true );

    m_pButtonStop = new QPushButton( m_pFrameButtons );
    m_pButtonStop->setFocusPolicy( QWidget::NoFocus );
    m_pButtonStop->setPixmap( locate( "data", "amarok/images/hi16-action-noatunstop.png" ) );
    m_pButtonStop->setFlat( true );

    m_pButtonNext = new QPushButton( m_pFrameButtons );
    m_pButtonNext->setFocusPolicy( QWidget::NoFocus );
    m_pButtonNext->setPixmap( locate( "data", "amarok/images/hi16-action-noatunforward.png" ) );
    m_pButtonNext->setFlat( true );

    QBoxLayout* lay = new QVBoxLayout( this );
    lay->addWidget( m_pFrame );

    QBoxLayout *lay3 = new QHBoxLayout( lay );
    m_pLay6 = new QVBoxLayout( lay3 );
    m_pLay6->addItem( new QSpacerItem( 0, 2 ) );

    QBoxLayout *lay7 = new QVBoxLayout( lay3 );
    QBoxLayout *lay5 = new QHBoxLayout( lay7 );

    m_pButtonLogo = new AmarokButton( this, locate( "data", "amarok/images/logo_new_active.png" ),
                                      locate( "data", "amarok/images/logo_new_inactive.png" ), false );
    lay5->addItem( new QSpacerItem( 4, 0 ) );
    lay5->addWidget( m_pButtonLogo );
    m_pTimeDisplayLabel = new QLabel( this );
    lay7->addItem( new QSpacerItem( 0, 3 ) );
    lay7->addWidget( m_pTimeDisplayLabel );

    QBoxLayout *lay4 = new QVBoxLayout( lay5 );
    m_pButtonPl = new AmarokButton( this, locate( "data", "amarok/images/pl_active.png" ),
                                    locate( "data", "amarok/images/pl_inactive.png" ), true );
    m_pButtonEq = new AmarokButton( this, locate( "data", "amarok/images/eq_active.png" ),
                                    locate( "data", "amarok/images/eq_inactive.png" ), true );
    lay4->addWidget( m_pButtonPl );
    lay4->addItem( new QSpacerItem( 0, 1 ) );
    lay4->addWidget( m_pButtonEq );
    lay4->addItem( new QSpacerItem( 2, 0 ) );

    lay->addItem( new QSpacerItem( 0, 5 ) );
    QBoxLayout* lay2 = new QHBoxLayout( lay );
    lay2->addItem( new QSpacerItem( 1, 0 ) );
    lay2->addWidget( m_pSlider );
    lay2->addItem( new QSpacerItem( 1, 0 ) );

    lay3->addItem( new QSpacerItem( 1, 0 ) );
    lay3->addWidget( m_pSliderVol );
    lay3->addItem( new QSpacerItem( 1, 0 ) );

    lay->addWidget( m_pFrameButtons );
    QBoxLayout *layButtons = new QHBoxLayout( m_pFrameButtons );
    layButtons->addWidget( m_pButtonPrev );
    layButtons->addWidget( m_pButtonPlay );
    layButtons->addWidget( m_pButtonPause );
    layButtons->addWidget( m_pButtonStop );
    layButtons->addWidget( m_pButtonNext );

    lay->setResizeMode( QLayout::FreeResize );
    layButtons->setResizeMode( QLayout::FreeResize );
    lay2->setResizeMode( QLayout::FreeResize );
    lay4->setResizeMode( QLayout::FreeResize );
    lay5->setResizeMode( QLayout::FreeResize );
    lay7->setResizeMode( QLayout::FreeResize );

    m_pFrame->setFixedSize( width(), 25 );
    m_pTimeDisplayLabel->setFixedSize( 9 * 12 + 2, 12 + 2 );

    // set up system tray
    m_pTray = new AmarokSystray( this, m_pActionCollection );
    m_pTray->show();
    QToolTip::add( m_pTray, i18n( "amaroK media player" ) );

    initTimeDisplay();
    initScroll();
    timeDisplay( false, 0, 0, 0 );

    // Register with DCOP
    if ( !kapp->dcopClient() ->isRegistered() )
    {
        kapp->dcopClient() ->registerAs( "amarok" );
        kapp->dcopClient() ->setDefaultObject( objId() );
    }
}


PlayerWidget::~PlayerWidget()
{}


// METHODS ----------------------------------------------------------------

void PlayerWidget::initScroll()
{
    m_pixmapWidth = 800;
    m_pixmapHeight = 20;

    m_pBgPixmap = new QPixmap( paletteBackgroundPixmap() ->convertToImage().copy( m_pFrame->x(),
                               m_pFrame->y(), m_pFrame->width(), m_pFrame->height() ) );

    m_pComposePixmap = new QPixmap( m_pFrame->width(), m_pixmapHeight );
    m_pScrollPixmap = new QPixmap( m_pixmapWidth, m_pixmapHeight );
    m_pScrollMask = new QBitmap( m_pixmapWidth, m_pixmapHeight );
    setScroll( i18n( "   welcome to amaroK   " ), " ", " " );

    m_sx = m_sy = 0;
    m_sxAdd = 1;
}


void PlayerWidget::initTimeDisplay()
{
    m_timeDisplayX = 190;
    m_timeDisplayY = 84;
    m_timeDisplayW = 12;

    m_pTimePixmap = new QPixmap( locate( "data", "amarok/images/numbers_transp.png" ) );

    m_pTimeBgPixmap = new QPixmap( paletteBackgroundPixmap() ->convertToImage()
                                   .copy( m_timeDisplayX, m_timeDisplayY, 9 * m_timeDisplayW, m_timeDisplayW ) );

    m_pTimeComposePixmap = new QPixmap( m_pTimeBgPixmap->width(), m_pTimeBgPixmap->height() );
}


void PlayerWidget::polish()
{
    QWidget::polish();
}


void PlayerWidget::setScroll( QString text, QString bitrate, QString samplerate )
{
    /* Update tray tooltip */
    if ( QToolTip::textFor( m_pTray ) != QString::null ) QToolTip::remove( m_pTray );
    QToolTip::add( m_pTray, text );

    m_nowPlaying = text;

    m_bitrate = bitrate;
    m_samplerate = samplerate;
    text.prepend( "   ***   " );

    m_pScrollMask->fill( Qt::color0 );
    QPainter painterPix( m_pScrollPixmap );
    QPainter painterMask( m_pScrollMask );
    painterPix.setBackgroundColor( Qt::black );
    painterPix.setPen( pApp->m_fgColor );
    painterMask.setPen( Qt::color1 );

    QFont font;
    font.setStyleHint( QFont::Helvetica );
    font.setFamily( "Helvetica" );
    font.setPointSize( 10 );
    //  font.setBold( true );
    painterPix.setFont( font );
    painterMask.setFont( font );

    painterPix.eraseRect( 0, 0, m_pixmapWidth, m_pixmapHeight );
    painterPix.drawText( 0, 0, m_pixmapWidth, m_pixmapHeight, Qt::AlignLeft || Qt::AlignVCenter, text );
    painterMask.drawText( 0, 0, m_pixmapWidth, m_pixmapHeight, Qt::AlignLeft || Qt::AlignVCenter, text );
    m_pScrollPixmap->setMask( *m_pScrollMask );

    QRect rect = painterPix.boundingRect( 0, 0, m_pixmapWidth, m_pixmapHeight,
                                          Qt::AlignLeft || Qt::AlignVCenter, text );
    m_scrollWidth = rect.width();

    // trigger paintEvent, so the Bitrate and Samplerate text gets drawn
    update();
}


void PlayerWidget::drawScroll()
{
    bitBlt( m_pComposePixmap, 0, 0, m_pBgPixmap );

    m_sx += m_sxAdd;
    if ( m_sx >= m_scrollWidth )
        m_sx = 0;

    int marginH = 4;
    int marginV = 3;
    int subs = 0;
    int dx = marginH;
    int sxTmp = m_sx;

    while ( dx < m_pFrame->width() )
    {
        subs = -m_pFrame->width() + marginH;
        subs += dx + ( m_scrollWidth - sxTmp );
        if ( subs < 0 )
            subs = 0;
        bitBlt( m_pComposePixmap, dx, marginV,
                m_pScrollPixmap, sxTmp, m_sy, m_scrollWidth - sxTmp - subs, m_pixmapHeight, Qt::CopyROP );
        dx += ( m_scrollWidth - sxTmp );
        sxTmp += ( m_scrollWidth - sxTmp ) ;

        if ( sxTmp >= m_scrollWidth )
            sxTmp = 0;
    }

    bitBlt( m_pFrame, 0, 0, m_pComposePixmap );
}


void PlayerWidget::timeDisplay( bool remaining, int hours, int minutes, int seconds )
{
    m_timeRemaining = remaining;
    m_timeHours = hours;
    m_timeMinutes = minutes;
    m_timeSeconds = seconds;

    update();
}


void PlayerWidget::drawTimeDisplay()
{
    bitBlt( m_pTimeComposePixmap, 0, 0, m_pTimeBgPixmap );

    int x = 0;
    int y = 0;

    if ( m_timeHours > 60 || m_timeHours < 0 )
        m_timeHours = 0;
    if ( m_timeMinutes > 60 || m_timeMinutes < 0 )
        m_timeMinutes = 0;
    if ( m_timeSeconds > 60 || m_timeSeconds < 0 )
        m_timeSeconds = 0;

    if ( m_timeRemaining )
        bitBlt( m_pTimeComposePixmap, x, y, m_pTimePixmap, 11 * m_timeDisplayW, 0, m_timeDisplayW );
    else
        bitBlt( m_pTimeComposePixmap, x, y, m_pTimePixmap, 12 * m_timeDisplayW, 0, m_timeDisplayW );

    x += m_timeDisplayW;

    bitBlt( m_pTimeComposePixmap, x, y, m_pTimePixmap, m_timeHours / 10 * m_timeDisplayW, 0, m_timeDisplayW );
    x += m_timeDisplayW;
    bitBlt( m_pTimeComposePixmap, x, y, m_pTimePixmap, m_timeHours % 10 * m_timeDisplayW, 0, m_timeDisplayW );
    x += m_timeDisplayW;
    bitBlt( m_pTimeComposePixmap, x, y, m_pTimePixmap, 10 * m_timeDisplayW, 0, m_timeDisplayW );
    x += m_timeDisplayW;
    bitBlt( m_pTimeComposePixmap, x, y, m_pTimePixmap, m_timeMinutes / 10 * m_timeDisplayW, 0, m_timeDisplayW );
    x += m_timeDisplayW;
    bitBlt( m_pTimeComposePixmap, x, y, m_pTimePixmap, m_timeMinutes % 10 * m_timeDisplayW, 0, m_timeDisplayW );
    x += m_timeDisplayW;
    bitBlt( m_pTimeComposePixmap, x, y, m_pTimePixmap, 10 * m_timeDisplayW, 0, m_timeDisplayW );
    x += m_timeDisplayW;
    bitBlt( m_pTimeComposePixmap, x, y, m_pTimePixmap, m_timeSeconds / 10 * m_timeDisplayW, 0, m_timeDisplayW );
    x += m_timeDisplayW;
    bitBlt( m_pTimeComposePixmap, x, y, m_pTimePixmap, m_timeSeconds % 10 * m_timeDisplayW, 0, m_timeDisplayW );

    //offset 1 pixel because of bounding box
    bitBlt( m_pTimeDisplayLabel, 1, 1, m_pTimeComposePixmap );
}


// EVENTS -----------------------------------------------------------------

void PlayerWidget::paintEvent( QPaintEvent * )
{
    erase( 20, 40, 120, 50 );
    QPainter pF( this );

    QFont font;
    font.setStyleHint( QFont::Helvetica );
    font.setFamily( "Helvetica" );
    font.setPointSize( 8 );
    pF.setFont( font );

    pF.setPen( pApp->m_fgColor );
    pF.drawText( 20, 40, m_bitrate );
    pF.drawText( 70, 40, m_samplerate );

    drawTimeDisplay();
}


void PlayerWidget::mouseReleaseEvent( QMouseEvent *e )
{
    if ( m_pTimeDisplayLabel->geometry().contains( e->pos() ) )
    {
        if ( pApp->m_optTimeDisplayRemaining )
            pApp->m_optTimeDisplayRemaining = false;
        else
            pApp->m_optTimeDisplayRemaining = true;
    }
}


void PlayerWidget::wheelEvent( QWheelEvent *e )
{
    e->accept();
    pApp->m_Volume += ( e->delta() * -1 ) / 18;

    if ( pApp->m_Volume < 0 )
        pApp->m_Volume = 0;
    if ( pApp->m_Volume > 100 )
        pApp->m_Volume = 100;

    pApp->slotVolumeChanged( pApp->m_Volume );
    m_pSliderVol->setValue( pApp->m_Volume );
}


void PlayerWidget::mousePressEvent( QMouseEvent *e )
{
    if ( e->button() == QMouseEvent::RightButton )
    {
        if ( !m_pPopupMenu )
        {
            m_pPopupMenu = new QPopupMenu( this );
            m_pPopupMenu->setCheckable( true );

            m_pActionCollection->action( "help_about_app" )->plug( m_pPopupMenu );
            m_pActionCollection->action( "help_contents"  )->plug( m_pPopupMenu );
            m_pActionCollection->action( "help_show_tip"  )->plug( m_pPopupMenu );

            m_pPopupMenu->insertSeparator();

            m_pActionCollection->action( "options_configure" )->plug( m_pPopupMenu );
            m_pActionCollection->action( "options_configure_keybinding" )->plug( m_pPopupMenu );
            m_pActionCollection->action( "options_configure_global_keybinding" )->plug( m_pPopupMenu );

            m_pPopupMenu->insertSeparator();

            m_pPopupMenu->insertItem( i18n( "Effects" ), pApp, SLOT( slotConfigEffects() ) );
            m_IdConfPlayObject = m_pPopupMenu->insertItem( i18n( "Configure PlayObject" ), this, SLOT( slotConfigPlayObject() ) );

            m_pPopupMenu->insertSeparator();

            m_IdRepeatTrack = m_pPopupMenu->insertItem( i18n( "Repeat Track" ), pApp, SLOT( slotSetRepeatTrack() ) );
            m_IdRepeatPlaylist = m_pPopupMenu->insertItem( i18n( "Repeat Playlist" ), pApp, SLOT( slotSetRepeatPlaylist() ) );
            m_IdRandomMode = m_pPopupMenu->insertItem( i18n( "Random Mode" ), pApp, SLOT( slotSetRandomMode() ) );

            m_pPopupMenu->insertSeparator();

            m_pActionCollection->action( "file_quit" )->plug( m_pPopupMenu );
        }

        m_pPopupMenu->setItemChecked( m_IdRepeatTrack, pApp->m_optRepeatTrack );
        m_pPopupMenu->setItemChecked( m_IdRepeatPlaylist, pApp->m_optRepeatPlaylist );
        m_pPopupMenu->setItemChecked( m_IdRandomMode, pApp->m_optRandomMode );

        if ( pApp->playObjectConfigurable() )
            m_pPopupMenu->setItemEnabled( m_IdConfPlayObject, true );
        else
            m_pPopupMenu->setItemEnabled( m_IdConfPlayObject, false );

        m_pPopupMenu->exec( e->globalPos() );
    }
}


void PlayerWidget::closeEvent( QCloseEvent *e )
{
    //KDE policy states we should hide to tray and not quit() when the close window button is
    //pushed for the main widget -mxcl
    //of course since we haven't got an obvious quit button, this is not yet a perfect solution..

    if( pApp->m_optShowTrayIcon )
    {
        KMessageBox::information( this,
            i18n( "<qt>Closing the main window will keep amaroK running in the system tray. "
                  "Use Quit from the popup-menu to quit the application.</qt>" ),
            i18n( "Docking in System Tray" ), "hideOnCloseInfo" );
        e->accept();
    }
    else if ( pApp->queryClose() )
        pApp->quit();
}


void PlayerWidget::moveEvent( QMoveEvent * )
{
//     You can get the frame sizes like so (found in Qt sources while looking for something else):
/*    int framew = geometry().x() - x();
    int frameh = geometry().y() - y();*/

    // Makes the the playlistwindow stick magnetically to the playerwindow 
    
/*    if ( pApp->m_pBrowserWin->isVisible() )
    {
        if ( ( frameGeometry().x() == pApp->m_pBrowserWin->frameGeometry().right() + 1 ) )
                ( e->oldPos().y() == pApp->m_pBrowserWin->frameGeometry().bottom() ) ||
                ( e->oldPos().x() + frameSize().width() + 0 == pApp->m_pBrowserWin->frameGeometry().left() ) ||
                ( e->oldPos().y() + frameSize().height() + 0 == pApp->m_pBrowserWin->frameGeometry().top() ) )
        {
            pApp->m_pBrowserWin->move( e->pos() + ( pApp->m_pBrowserWin->pos() -  e->oldPos() ) );
            pApp->m_pBrowserWin->move( e->pos() + ( pApp->m_pBrowserWin->pos() -  e->oldPos() ) );
        }
    }*/
}


void PlayerWidget::createVis()
{
    if ( pApp->m_optVisCurrent == 0 )
        m_pVis = new VisWidget( this );
    else
        m_pVis = new VisDistortWidget( this );

    m_pVis->setFixedSize( 168, 50 );
    m_pLay6->addWidget( m_pVis );
    connect( m_pVis, SIGNAL( clicked() ), this, SLOT( visClicked() ) );

    m_pVis->show();
}


void PlayerWidget::visClicked()
{
    delete m_pVis;

    pApp->m_optVisCurrent++;
    pApp->m_optVisCurrent %= VIS_COUNT;

    createVis();
}


// SLOTS ---------------------------------------------------------------------

void PlayerWidget::slotConfigShortcuts()
{
    KKeyDialog keyDialog( true );

    keyDialog.insert( m_pActionCollection, "Player Window" );
    keyDialog.insert( pApp->m_pBrowserWin->m_pActionCollection, "Playlist Window" );

    keyDialog.configure();
}


void PlayerWidget::slotConfigGlobalShortcuts()
{
    KKeyDialog::configure( pApp->m_pGlobalAccel, true, 0, true );
}


void PlayerWidget::slotCopyClipboard()
{
    QListViewItem * currentTrack = pApp->m_pBrowserWin->m_pPlaylistWidget->currentTrack();

    if ( currentTrack )
    {
        QClipboard * cb = QApplication::clipboard();
        cb->setText( currentTrack->text( 0 ) );
    }
}


void PlayerWidget::slotConfigPlayObject()
{
    if ( pApp->m_pPlayObject && !m_pPlayObjConfigWidget )
    {
        m_pPlayObjConfigWidget = new ArtsConfigWidget( pApp->m_pPlayObject->object(), this );

        connect( m_pPlayObjConfigWidget, SIGNAL( destroyed() ), this, SLOT( slotConfigWidgetDestroyed() ) );
        m_pPlayObjConfigWidget->show();
    }
}


void PlayerWidget::slotConfigWidgetDestroyed()
{
    m_pPlayObjConfigWidget = NULL;
}


void PlayerWidget::slotUpdateTrayIcon( bool visible )
{
    if ( visible )
    {
        m_pTray->show();
    }
    else
    {
        m_pTray->hide();
    }
}


void PlayerWidget::show()
{
    emit sigRestored();
    QWidget::show();
}


void PlayerWidget::hide()
{
    emit sigMinimized();
    QWidget::hide();
}


/* DCOP signals - first try at it */

void PlayerWidget::play()
{
    pApp->slotPlay();
}


void PlayerWidget::stop()
{
    pApp->slotStop();
}


void PlayerWidget::next()
{
    pApp->slotNext();
}


void PlayerWidget::prev()
{
    pApp->slotPrev();
}


void PlayerWidget::pause()
{
    pApp->slotPause();
}


QString PlayerWidget::nowPlaying()
{
    return m_nowPlaying;
}

bool PlayerWidget::isPlaying()
{
    return pApp->isPlaying();
}


#include "playerwidget.moc"
